
#include <iostream.h>
#include <limits.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <float.h>
#include "bignum.h"


//***********************************************************
//		Minnesallokering
//-----------------------------------------------------------
//	Denna rutin allokerar minne genom att anropa
// malloc i stdlib.h med den storlek talet skall ha. Om
// malloc returnerar att ngot gick snett (t.ex minnesbrist)
// s skrivs ett felmeddelande ut och krningen avslutas.
// Gr allt som det ska s returneras en pekare till det 
// allokerade minnesomrdet.
//***********************************************************

char * Malloc(size_t size)
	{
	char * a;
	if(size == 0) return 0;
	a=(char *)malloc(size);
	if(a==0)
		{
		printf("Out of memory.\n");
		exit(1);
		}
	return a;
	}


//***********************************************************
//		Reallokering av minne
//-----------------------------------------------------------
//	Denna rutin kopierar minnesinnehllet fran en
// tidigare allokerat omrde till en annat omrde i minnet.
// Den anvnder sig av realloc rutinen i stdlib.h, och
// anvnds nar man behver mer plats utan att forstra det 
// ursprungliga innehllet.
//***********************************************************

char * Realloc(char *ptr,size_t size)
	{
	char * a;
	a=(char *)realloc((void *)ptr,size);
	if(a==0 && size !=0)
		{
		printf("Out of memory.\n");
		exit(1);
		}
	return a;
	}
	

//***********************************************************
//		Konstruktorer och destruktor
//-----------------------------------------------------------
//	Konstruktorerna fungerar s att, argumentet de
// anropas med styr hur det stora talet konstrueras. Frst
// har vi konstruktorn fr ett tomt tal det fr lngden 1 och
// vrdet 0. Drefter har vi destruktorn fr stora tal som
// anropar "free" i stdlib.h. Sedan r det konstruktorer fr
// stora tal som skapas frn heltal, andra stora tal (anvnds
// vid kopiering, return och liknande) och stora tal som
// skapas frn strngar. Talets egenskaper r en pekare till
// den frsta siffran i talet, talets lngd och talets
// tecken.
//***********************************************************

Bignum::Bignum()
	{
	length=1;
	digits=Malloc(1);
	digits[0]=0;
	negative=false;
	}
	
Bignum::~Bignum()
	{
	free(digits);
	}
	
Bignum::Bignum(int a)
	{
	int i=0;
	if (a<0)
		{
		negative=true;
		a=-a;
		}
	else
		{
		negative = false;
		}
	length=(CHAR_BIT*sizeof(int)+1)/3;
	digits=Malloc(length);
	for(i=length-1;i>=0;i--)
		{
		digits[i]=a%10;
		a/=10;
		}
	
	normalize();
	}
	
Bignum::Bignum(double b)
	{
	double a=b;
	int i=0;
	if (a<0)
		{
		negative=true;
		a=-a;
		}
	else
		{
		negative = false;
		}
	if(a==0)
		{
		length=1;
		digits=Malloc(1);
		digits[0]=0;
		return;
		}
	length=int(log10(a)+1);
	digits=Malloc(length);
	for(i=length-1;i>=0;i--)
		{
		digits[i]=(int)fmod(a,10.0);
		a/=10.0;
		}
	normalize();
	}
	
Bignum::Bignum(const Bignum& b)
	{
	negative=b.negative;
	length=b.length;
	if(length>0)
		{
		digits=Malloc(length);
		memcpy(digits,b.digits,length);
		}
	else
		{
		digits=Malloc(1);
		length=1;
		}
		
	}
	
	
Bignum::Bignum(char *s)
	{
	int i,t,m;
	t=strlen(s);
	if(t==0)
		{
		digits=Malloc(1);
		digits[0]=0;
		return;
		}
	digits=Malloc(t);
	length=t;
	negative=false;
	if(s[0]=='-')
		{
		negative=true;
		m=1;
		digits[0]=0;
		}
	else
		{
		m=0;
		}
	for(i=m;i<t;i++)
		{
		if(s[i]>='0' && s[i]<='9')
			{
			digits[i]=s[i]-'0';
			}
		else
			{
			length=i;
			digits=Realloc(digits,i);
			break;
			}
		}
	normalize();	
	}


//***********************************************************
// Operatorer fr omvandling av ett Bignum till en int eller double.
// Om Bignum r fr stor fr att kunna representeras i den givna
// typen skrivs ett felmeddelande ut och programmet avslutas.
//***********************************************************

Bignum::operator int()
	{
	int a;
	int i;
	if ((*this > Bignum(INT_MAX)) || (*this < Bignum(INT_MIN)))
		{
		printf("Overflow\n");
		exit(1);
		}		
	a=0;
	for(i=0;i<length;i++)
		{
		a*=10;
		a=a+digits[i];
		}
	if(negative) a=-a;
	return a;
	}
	
Bignum::operator double()
	{
	double a;
	int i;
	if (abs(*this) > DBL_MAX)
		{
		printf("Overflow\n");
		exit(1);
		}		
	a=0.0;
	for(i=0;i<length;i++)
		{
		a*=10.0;
		a=a+digits[i];
		}
	if(negative) a=-a;
	return a;
	}

//************************************************************
//		Tilldelningsoperatorn
//------------------------------------------------------------
//	Tilldelningsoperatorn tar ett bignum som argument,
// ser till att det allokeras minne fr ett nytt tal med samma
// lngd som argumentet, att talet fr samma tecken och lngd
// som argumentet och kopierar sedan argumentets siffror till
// det nya talet. Sjlva kopieringen skts med standardfunktionen
// memcpy.
//************************************************************

Bignum& Bignum::operator=(const Bignum& b)
	{
	free(digits);
	negative=b.negative;
	length=b.length;
	if(length>0)
		{
		digits=Malloc(length);
		memcpy(digits,b.digits,length);
		}
	else
		{
		digits=0;
		length=0;
		}
	return *this;
	}


//************************************************************
//		Jmfrelseoperatorer (likhet/olikhet)
//------------------------------------------------------------
//	Dessa tv operatorer kollar om tv tal ar lika/olika.
// Frst jmfr rutinen talens tecken. r dessa lika
// fortstter den att kolla talens lngd. r dessa lika jmfor
// rutinen talen siffra for siffra och hoppar ut s snart 
// olikhet har ptrffats.
//************************************************************

int operator==(const Bignum& b1,const Bignum& b2)
	{
	int i;
	if(b1.negative != b2.negative) return 0;
	if(b1.length==b2.length)
		{
		for(i=0;i<b1.length;i++)
			{
			if(b1.digits[i]!=b2.digits[i]) return 0;
			}
		return 1;
		}
	else
		return 0;
	}

int operator!=(const Bignum& b1,const Bignum& b2)
	{
	int i;
	if(b1.negative!=b2.negative) return 1;
	if(b1.length==b2.length)
		{
		for(i=0;i<b1.length;i++)
			{
			if(b1.digits[i]!=b2.digits[i]) return 1;
			}
		return 0;
		}
	else
		return 1;
	}


//************************************************************
//		Jmforelseoperatorer (strre n/mindre n)
//------------------------------------------------------------
//	Med hjlp av dessa rutiner kan man kolla om ett tal r
// strre n, mindre n ,strre n eller lika med eller mindre
// n eller lika med ett annat tal. Rutinerna tar tv tal som
// argument och brjar med att kolla tecken. Drefter jmfrs
// (om ndvandigt) talens lngder, och r aven dessa lika
// jmfors talen siffra for siffra.
//************************************************************

int operator<(const Bignum& b1,const Bignum& b2)
	{
	int i;
	if(b1.negative == true && b2.negative==false) return 1;
	if(b1.negative == false && b2.negative== true) return 0;
	if(b1.negative == true && b2.negative == true)
		{
		if(b1.length > b2.length) return 1;
		if(b1.length < b2.length) return 0;
		for(i=0;i<b1.length;i++)
			{
			if(b1.digits[i]>b2.digits[i]) return 1;
			if(b1.digits[i]<b2.digits[i]) return 0;
			}
		return 0;
		}
	else  // Bda talen > 0
		{
		if(b1.length > b2.length) return 0;
		if(b1.length < b2.length) return 1;
		for(i=0;i<b1.length;i++)
			{
			if(b1.digits[i]>b2.digits[i]) return 0;
			if(b1.digits[i]<b2.digits[i]) return 1;
			}
		return 0;
		}
	}

int operator>(const Bignum& b1,const Bignum& b2)
	{
	int i;
	if(b1.negative == true && b2.negative==false) return 0;
	if(b1.negative == false && b2.negative== true) return 1;
	if(b1.negative == true && b2.negative == true)
		{
		if(b1.length > b2.length) return 0;
		if(b1.length < b2.length) return 1;
		for(i=0;i<b1.length;i++)
			{
			if(b1.digits[i]>b2.digits[i]) return 0;
			if(b1.digits[i]<b2.digits[i]) return 1;
			}
		return 0;
		}
	else  // Bda talen > 0
		{
		if(b1.length > b2.length) return 1;
		if(b1.length < b2.length) return 0;
		for(i=0;i<b1.length;i++)
			{
			if(b1.digits[i]<b2.digits[i]) return 0;
			if(b1.digits[i]>b2.digits[i]) return 1;
			}
		return 0;
		}
	}

int operator>=(const Bignum& b1,const Bignum& b2)
	{
	int i;
	if(b1.negative == true && b2.negative==false) return 0;
	if(b1.negative == false && b2.negative== true) return 1;
	if(b1.negative == true && b2.negative == true)
		{
		if(b1.length > b2.length) return 0;
		if(b1.length < b2.length) return 1;
		for(i=0;i<b1.length;i++)
			{
			if(b1.digits[i]>b2.digits[i]) return 0;
			if(b1.digits[i]<b2.digits[i]) return 1;
			}
		return 1;
		}
	else  // Bda talen > 0
		{
		if(b1.length > b2.length) return 1;
		if(b1.length < b2.length) return 0;
		for(i=0;i<b1.length;i++)
			{
			if(b1.digits[i]<b2.digits[i]) return 0;
			if(b1.digits[i]>b2.digits[i]) return 1;
			}
		return 1;
		}
	}

int operator<=(const Bignum& b1,const Bignum& b2)
	{
	int i;
	if(b1.negative == true && b2.negative==false) return 1;
	if(b1.negative == false && b2.negative== true) return 0;
	if(b1.negative == true && b2.negative == true)
		{
		if(b1.length > b2.length) return 1;
		if(b1.length < b2.length) return 0;
		for(i=0;i<b1.length;i++)
			{
			if(b1.digits[i]<b2.digits[i]) return 0;
			if(b1.digits[i]>b2.digits[i]) return 1;
			}
		return 1;
		}
	else  // Bda talen > 0
		{
		if(b1.length > b2.length) return 0;
		if(b1.length < b2.length) return 1;
		for(i=0;i<b1.length;i++)
			{
			if(b1.digits[i]>b2.digits[i]) return 0;
			if(b1.digits[i]<b2.digits[i]) return 1;
			}
		return 1;
		}
	}


//*************************************************************
//		+=, -=, *= och /= operatorerna
//-------------------------------------------------------------
//	Dessa operatorer tar ett bignum som argument, skapar
// ett tal, a, anropar lmplig operator (+, -, * eller /), skriver
// ver och returnerar objektet den anropades fr.
//*************************************************************

Bignum& Bignum::operator+=(const Bignum& b)
	{
	Bignum a;
	a= *this + b;
	*this = a;
	return *this;
	}
	
Bignum& Bignum::operator-=(const Bignum& b)
	{
	Bignum a;
	a= *this - b;
	*this = a;
	return *this;	
	}

Bignum& Bignum::operator*=(const Bignum& b)
	{
	Bignum a;
	a= *this * b;
	*this = a;
	return *this;
	}
	
Bignum& Bignum::operator/=(const Bignum& b)
	{
	Bignum a;
	a= *this / b;
	*this = a;
	return *this;
	}


//*************************************************************
//		Operatorn plus
//-------------------------------------------------------------
//	Operatorn plus anvands fr att addera tv stora tal.
// Den anropas med tv bignum som argument, och returnerar ett
// bignum som resultat. Frst hanterar man argumentens tecken
// eftersom grundalgoritmen bara hanterar positiva tal.
// Drefter jmfors talen fr att avgra vilket som r strst,
// och ett tal av typen bignum med lngden (lngden av det
// lngsta talet + 1) allokeras som resultat. Sjlva
// additionsalgoritmen gr till s att man adderar de stora
// talen siffra fr siffra och kollar om det blir ngon
// carrysiffra, som skall fras over till nsta position i
// talet. Nr additionen r klar anropas rutinen normalize fr
// att skala av eventuella nollor i brjan p talet.
//************************************************************

Bignum operator+(const Bignum& b1,const Bignum& b2)
	{
	int i,t;
	int maxl,minl;
	const Bignum *max,*min;
	Bignum a;
	if(b1.negative && b2.negative) return -(-b1 + -b2);
	if(b1.negative && !b2.negative) return b2 - (-b1);
	if(!b1.negative && b2.negative) return b1- (-b2);
	if(b1 > b2)
		{
		max= &b1;
		min= &b2;
		}
	else
		{
		max= &b2;
		min= &b1;
		}
	maxl=max->length;
	minl=min->length;
	a.digits=Malloc(maxl+1);
	for(i=0;i<=maxl;i++) a.digits[i]=0;
	a.length=maxl+1;
	for(i=1;i<=minl;i++)
		{
		t=max->digits[maxl-i]+min->digits[minl-i]+a.digits[maxl+1-i];
		a.digits[maxl+1-i]=t%10;
		a.digits[maxl-i]=t/10;
		}
	for(i=minl+1;i<=maxl;i++)
		{
		t=max->digits[maxl-i]+a.digits[maxl+1-i];
		a.digits[maxl+1-i]=t%10;
		a.digits[maxl-i]=t/10;
		}
	a.normalize();
	return a;		
	}


//************************************************************
//		Operatorn binrt minus
//------------------------------------------------------------
//	Operatorn minus subtraherar ett stort tal frn ett
// annat. Den anropas med tv bignum som argument och ger ett
// bignum som resultat. Frst hanteras eventuella
// minustecken i argumenten eftersom grundalgoritmen
// bara hanterar positiva tal. Sedan jmfor rutinen de tv
// talen fr att avgra vilket av dem som r strst.
// Algoritmen krver nmligen att resultatet >= 0.
// Algoritmen r den gamla vanliga, man gr igenom talen siffra
// fr siffra , entalen frst sedan tiotalen osv. Om den vre
// siffran minus den undre blir negativt lnar man frn nrmast
// hgre siffra.
//************************************************************

Bignum operator-(const Bignum& b1,const Bignum& b2)
	{
	int i,t,m;
	int maxl,minl;
	const Bignum *max,*min;
	Bignum a;
	if(b1.negative && b2.negative) return -b2 - -b1;
	if(b1.negative && !b2.negative) return -(-b1 +b2);
	if(!b1.negative && b2.negative) return b1 + -b2;
	if(b1 > b2)
		{
		max= &b1;
		min= &b2;
		a.negative=false;
		}
	else
		{
		max= &b2;
		min= &b1;
		a.negative=true;
		}
	
	maxl=max->length;
	minl=min->length;
	a.digits=Malloc(maxl);
	a.length=maxl;
	for(i=0;i<maxl;i++)a.digits[i]=0;
	m=0;
	for(i=1;i<=minl;i++)
		{
		t=max->digits[maxl-i]-min->digits[minl-i]-m;
		if(t < 0)
			{
			t=t+10;
			m=1;	
			}		
		else
			{
			m=0;		
			}
		a.digits[maxl-i]=t;
		}
	for(i=minl+1;i<=maxl;i++)
		{
		t=max->digits[maxl-i]-m;
		if(t < 0)
			{
			t=t+10;
			m=1;	
			}		
		else
			{
			m=0;		
			}
		a.digits[maxl-i]=t;
		}
	a.normalize();
	return a;
	}


//************************************************************
//		Operatorn unrt minus
//------------------------------------------------------------
//	Operatorn unrt minus tar ett bignum som argument och
// byter tecken p det, och returnerar det.
//************************************************************
	
Bignum operator-(const Bignum& b)
	{
	Bignum a(b);
	if(a.negative)	
		a.negative=false;
	else
		a.negative=true;
	
	return a;
	}
		

//***********************************************************
//		Operatorn gnger
//-----------------------------------------------------------
//	Operatorn gnger tar tv bignum som argument och ger
// tillbaka ett bignum som resultat. Frst hanteras
// eventuella minustecken i argumenten, eftersom
// grundalgoritmen bara hanterar positiva operander. Sedan
// jmfr man vilket av talen som r strst och allokerar
// ett tal som har lika mnga siffror som de tv argumenten
// tillsammans (resultatet) och ett tal som har en siffra mer
// n det strsta talet (temporr variabel).
// Sjlva multiplikationsalgoritmen gr till s att man 
// multiplicerar en siffra i det mindre talet med hela det
// strre talet och lagrar resultatet i en temporr vektor
// som frskjuts ett antal steg t hger lika med antalet siffror
// i det mindre talet som redan r frdigbehandlade och
// adderas  sedan till resultatet. Sedan tar man nsta
// siffra i det minsta talet och multiplicerar med hela det
// strsta talet o.s.v.
//***********************************************************

Bignum operator*(const Bignum& b1,const Bignum& b2)
	{
	int i,j,t,m,mult;
	int maxl,minl;
	const Bignum *max,*min;
	Bignum a;
	char *tmp;
	if(b1.negative && b2.negative) return -b1 * -b2;
	if(b1.negative && !b2.negative) return -b1 * b2;
	if(!b1.negative && b2.negative) return b1 * -b2;
	
	if(b1 > b2)
		{
		max = &b1;
		min = &b2;
		}
	else
		{
		max = &b2;
		min = &b1;
		}
	maxl = max->length;
	minl = min->length;
	
	a.digits=Malloc(maxl+minl);
	for(i=0;i<maxl+minl;i++) a.digits[i]=0;
	a.length=maxl+minl;
	tmp=Malloc(maxl+1);
	
	for(i=1;i<=minl;i++)
		{
		mult=min->digits[minl-i];
		t=0;m=0;
		for(j=1;j<=maxl;j++)
			{
			t=mult*max->digits[maxl-j]+m;
			m=t/10;
			tmp[maxl+1-j]=t%10;
			tmp[maxl-j]=m;
			}
		t=0;m=0;
		for(j=0;j<=maxl;j++)
			{
			t=tmp[maxl-j]+a.digits[maxl+minl-i-j]+m;
			m=t/10;
			a.digits[maxl+minl-i-j]=t%10;
			}
		}	
	a.normalize();
	free(tmp);
	return a;
	}


Bignum operator/(const Bignum& b1,const Bignum& b2)
	{
	Bignum a,b,c;
	int i,t;
	if(b1.negative && b2.negative) return -b1 / -b2;
	if(b1.negative && !b2.negative) return -b1 / b2;
	if(!b1.negative && b2.negative) return b1 / -b2;
	if(b1 < b2) return 0;
	if(b1==b2) return 1;
	if(b2==1) return b1;
	a.digits=Malloc(b1.length);
	a.length=b1.length;
	b=0;
	for(i=0;i<b1.length;i++)
		{	
		b*=10;
		b=b+b1.digits[i];
		if(b>=b2)
			{
			c=b2;
			t=0;
			while(c<=b)
				{
				c+=b2;
				t++;
				}
			a.digits[i]=t;
			b=b-t*b2;
			}
		else
			{
			a.digits[i]=0;
			}
		}
	a.normalize();
	return a;
	}
	
	
	
	
	
	


//***********************************************************
//		Normaliseringsrutinen
//-----------------------------------------------------------
//	Normaliserings rutinen skalar bort alla nollor i
// brjan p talet. Frst kollar man om talet r lngre an
// en siffra. (r talet noll skall nollan finnas kvar.)
// Sedan kr vi igenom talet tills vi hittar ett nollskillt
// element. Drefter kopierar rutinen de kvarvarande
// elementen till en vektor av rtt storlek.
//***********************************************************

void Bignum::normalize()
	{
	int i,j;
	for(i=0;i<length-1;i++)
		{
		if(digits[i]!=0)
			{
			break;
			}
		}
	if(i==length-1)
		{
		if(digits[length-1]==0)
			{
			digits=Realloc(digits,1);
			digits[0]=0;
			negative=false; // Inga negativa nollor, tack.
			length=1;
			return;
			}
		}
	if(i==0) return;
	for(j=i;j<length;j++)
		{
		digits[j-i]=digits[j];
		}
	digits=Realloc(digits,length-i);
	length=length-i;
	}
	

//***********************************************************
//		Utskriftsrutinen
//-----------------------------------------------------------
//	Utskriftsrutinen tar ett stort tal som argument och
// skriver ut det. Det kollar om det skall skrivas ut ett 
// minustecken, och sedan skriver rutinen ut siffrorna som
// ASCII-tecken, d.v.s. den lgger till ASCII-koden fr '0'
// till siffran.
//***********************************************************
	
void print(const Bignum& a)
	{
	int i;
	if(a.negative) putchar('-');
	for(i=0;i<a.length;i++)
		putchar(a.digits[i]+'0');
	}


//***********************************************************
// abs() returnerar absolutbeloppet av sitt argument	
//***********************************************************
Bignum abs(Bignum a)
	{
	if(a < 0)
		return -a;
	else
		return a;
	}
	
	
	
	
	
	